%% This file estimates the spillover effect of QE in the US on the Canadian economy. 
% "International Transmission Channels of U.S. Quantitative Easing: Evidence from Canada"
% - Tatjana Dahlhaus, Kristina Hess and Abeer Reza
% JMCB MS14-430
%% *************************************************************************
% This code loads the following model: 
% XUS = LAMBDA_US^star*FUS
% XCAN = LAMBDA_US*FUS + LAMBDA_CAN*FCAN
% And estimates a block-restricted VAR from the model
% [FUS FCAN] = [BUS 0 ; BCANUS BCAN]*[LFUS LFCAN]
% And calculates the difference between the counter-factual and actual 
% data, taking into account bootstrapped uncertainty on the VAR to factor 
% conversion beta-coefficients. 

%% *************************************************************************
clear all;
clc;

close all;

% *****************************************************
% PARAMETER VALUES
NF_US = 4; %Number of factors in US
NF_CAN = 4; %Number of factors in CAN
lags = 1; %Number of lags in VAR
begs = 3; % To allow for transformation codes
horz = 20; %Forecast horizon. Enddate is 2013Q3. horz=20 means start from 2008Q3 
cdate = 19; % Counterfactual starts 2008Q4 (Baumeister and Benati)
nrep = 1000;   % Number of bootstrap iterations

% *****************************************************


%% Read in Data: 
% *******************************************************
% Read in the data: 
% ------------------------------------------------------
[DUS,MNEMUS] = xlsread('US_Data.xls'); 
[DCAN,MNEMCAN] = xlsread('CAN_Data.xls'); 
MNEMUS = MNEMUS(1,:);
MNEMCAN = MNEMCAN(1,:);

% Transformation code: 
US_trans = DUS(1,:); 
CAN_trans = DCAN(1,:); 

% Index of slow-moving variables: 
US_slow = DUS(2,:);
CAN_slow = DCAN(2,:);

% Re-define data matrix: 
DUS = DUS(begs:end,:);
DCAN = DCAN(begs:end,:);
[TUS NUS] = size(DUS);
[TCAN NCAN] = size(DCAN);

US_index = [1:NUS];
CAN_index = [1:NCAN];

US_neg = zeros(1,NUS);
CAN_neg = zeros(1,NCAN);

for ii = 1:NUS;
    if min(DUS(:,ii))<0;
        US_neg(ii) = 1;
    end;
end;

for ii = 1:NCAN;
    if min(DCAN(:,ii))<0;
        CAN_neg(ii) = 1;
    end;
end;



%% >>>>>>>>> DECLARE VARIABLES OF INTEREST <<<<<<<<
% --------------------------------------------
dim1 = 4; 
dim2 = 3;
% --------------------------------------------

USVAR = {'USrGDP','USPCEprices','US10ySpread','USrC','USEmpNonfarm','USSP500index','USrI','USConfidence','USCommCILoans','USCommLoans','USNEER','USToT','QE2','USWillingnessToLend','USCorporateSpread','USrExports','USrImports'};
CANVAR = {'CANrGDP','CANCPI','CAN10ySpread','CANrC','CANEmp','CANTSXindex','CANrI','CANConfidence','CANBankLoans','CANBankLoans','USDzCAD','CANToT','CANrExportstoUS','Brent','CAN3mCorpSpread','CANrExports','CANrImports','CANrExportstoUS','CANrImportsFromUS'};
CANGDPVAR = {'CANryGoods', 'CANryMin', 'CANryMan', 'CANrySvc', 'CANryUtl','CANryCon','CANryRet','CANryArt','CANryAcc'};

EXPVAR = {'CANrGoodsExports','CANrForExports','CANrEnergyExports','CANrMetalExports','CANrChemExports','CANrIndExports','CANrElecExports','CANrAutoExports','CANrConsGoodsExports','CANrServiceExports'};
IMPVAR = {'CANrGoodsImports','CANrBldgImports','CANrEnergyImports','CANrMetalImports','CANrChemImports','CANrIndImports','CANrElecImports','CANrAutoImports','CANrConsGoodsImports','CANrServiceImports'};
NETVAR = {'Goods','Forestry','Energy','Metal Products','Chemical','Industrial M&E','Electrical','Auto','Cons Goods','Services'};


USVARNAM = {'US GDP (%)','PCE price (%)','10 Yr Spread','Consumption (%)','Employment (%)','S&P 500 index (%)','Investment (%)','Confidence','Bank Loans (%)','Net Exports (% GDP)','Exchange Rate (%)','Terms of Trade (%)','QE (%)','Ease of Lending','Corporate Spread'};
CANVARNAM = {'CAN GDP (%)','CPI (%)','10 Yr Spread','Consumption (%)','Employment (%)','TSX index (%)','Investment (%)','Confidence','Bank Loans (%)','Net Exports (% GDP)','USD per CAD','Terms of Trade (%)','Net Exports to US  (% GDP)','Brent Price (%)','CAN Corp Spread'};
CANYNAM = {'Goods', 'Mining', 'Manufactuing', 'Services', 'Utility', 'Construction', 'Retail trade', 'Arts', 'Accommodation'};

XUS_index = [];
 for jj = 1:size(USVAR,2);
    for ii = 1:NUS;
        if strcmp(MNEMUS(ii),USVAR(jj)) == 1;
            XUS_index(jj) = US_index(ii);
        end;
        if strcmp(MNEMUS(ii),'USEffFedFundsRate') == 1;
            FF_index = US_index(ii);
        end;
        if strcmp(MNEMUS(ii),'QE2') == 1;
            QE_index = US_index(ii);
        end;

    end;
end;

XCAN_index = [];
 for jj = 1:size(CANVAR,2);
    for ii = 1:NCAN;
        if strcmp(MNEMCAN(ii),CANVAR(jj)) == 1;
            XCAN_index1(jj) = CAN_index(ii);
        end;
    end;
end;

for jj = 1:size(EXPVAR,2);
    for ii = 1:NCAN;
        if strcmp(MNEMCAN(ii),EXPVAR(jj)) == 1;
            XCAN_index2(jj) = CAN_index(ii);
        end;
    end;
end;

for jj = 1:size(IMPVAR,2);
    for ii = 1:NCAN;
        if strcmp(MNEMCAN(ii),IMPVAR(jj)) == 1;
            XCAN_index3(jj) = CAN_index(ii);
        end;
    end;
end;

for jj = 1:size(CANGDPVAR,2);
    for ii = 1:NCAN;
        if strcmp(MNEMCAN(ii),CANGDPVAR(jj)) == 1;
            XCAN_index4(jj) = CAN_index(ii);
        end;
    end;
end;


XCAN_index = [XCAN_index1 XCAN_index4 XCAN_index2 XCAN_index3];

N1 = size(XCAN_index1,2);
N2 = size(XCAN_index1,2) + size(XCAN_index4,2);
N3 = size(XCAN_index1,2) + size(XCAN_index4,2) + size(XCAN_index2,2);

% --------------------------------------------

%% Data transformation
% -----------------------------------------------------

US_trans(QE_index) = 2;

% Transform using code given in sheet. 
for jj = 1:NUS;
    if US_trans(jj)==1; % No transformation
        XXUS(:,jj) = DUS(:,jj);
    elseif US_trans(jj)==2; % Difference
        XXUS(:,jj) = DUS(:,jj) - lagmatrix(DUS(:,jj),1);
    elseif US_trans(jj)==4; % Log Transformation
        XXUS(:,jj) = log(DUS(:,jj));
    elseif US_trans(jj)==5; % Log difference
        XXUS(:,jj) = log(DUS(:,jj)) - lagmatrix(log(DUS(:,jj)),1);
    end;
end;

for jj = 1:NCAN;
    if CAN_trans(jj)==1; % No transformation
        XXCAN(:,jj) = DCAN(:,jj);
    elseif CAN_trans(jj)==2; % Difference
        XXCAN(:,jj) = DCAN(:,jj) - lagmatrix(DCAN(:,jj),1);
    elseif CAN_trans(jj)==4; % Log Transformation
        XXCAN(:,jj) = log(DCAN(:,jj));
    elseif CAN_trans(jj)==5; % Log difference
        XXCAN(:,jj) = log(DCAN(:,jj)) - lagmatrix(log(DCAN(:,jj)),1);
    end;
end;

% Parse the data: 
XXUS(1,:) = [];
XXCAN(1,:) = [];

% Redefine T and N:
[TUS NUS] = size(XXUS);
[TCAN NCAN] = size(XXCAN);

%% Extract Principal Components: 
% ****************************************************************

% Standardize for principal components: 
XUS = (XXUS - repmat(mean(XXUS,1),TUS,1))./repmat(std(XXUS,1),TUS,1);
XCAN = (XXCAN - repmat(mean(XXCAN,1),TCAN,1))./repmat(std(XXCAN,1),TCAN,1);


% Extract Principal Components from Slow-moving Data for US: 
slowindex = [];
fastindex = [];
for ii = 1:NUS;
    if US_slow(ii) == 1;
        slowindex = [slowindex; US_index(ii)];
    else
        fastindex = [fastindex; US_index(ii)];
    end;
end;

[FUS_all, LUS_all] = extract(XUS,NF_US);
XUS_slow = XUS(:,slowindex);
[FUS_slow, LUS_slow] = extract(XUS_slow,NF_US);

% Define the fast-moving data for US: 
FFRUS = XUS(:,FF_index);
QEUS = XUS(:,QE_index);
Y = [FFRUS  QEUS];
NY = size(Y,2);

FUS = facrot(FUS_all, Y, FUS_slow);
FYUS = [FUS Y];

% Extract Principal Components from Full Data: 
[FCAN0, LCAN] = extract(XCAN,NF_CAN);
FCANI = [];
FCANHAT(:,:,1) = FCAN0;
 for ll = 1:50; 
    XCANHAT(:,:,ll) = facrot(XCAN, FYUS, FCANHAT(:,:,ll));
    [FCANHAT(:,:,ll+1), LCAN] = extract(XCANHAT(:,:,ll), NF_CAN);
 end;

 FCAN = FCANHAT(:,:,end);

%% Estimate the FAVAR: 
% ************************************************************************
% Define the Variables of interest: 

XRUS = XUS(:,XUS_index); % Variable of interest
BRUS = olssvd(XRUS, FYUS);   % BETA to transform Factors back to XR
UUS = XRUS - FYUS*BRUS;

XRCAN = XCAN(:,XCAN_index); % Variable of interest
BRCAN = olssvd(XRCAN, [FYUS FCAN]);   % BETA to transform Factors back to XR
UCAN = XRCAN - [FYUS FCAN]*BRCAN;


% -------------------------------------------------------------------------
% Define the VAR: 

FAC = [FYUS FCAN];

% Position of QE variable in the VAR: 
NQE = NF_US+NY;
NF = size(FAC,2);

LFAC = [lagmatrix(FAC,1:lags)];
LFAC(1:lags,:)=[];
FAC(1:lags,:)=[];

XRCAN(1:lags,:) = [];
UCAN(1:lags,:) = [];
XRUS(1:lags,:) = [];
UUS(1:lags,:) = [];

% Estimate the VAR: 
% ---------------------------------------------
% BB(i,j) gives the coefficients of FYUS(:,j) on LFYUS(:,i)
% Example: AR(2)
% [xt yt xt-1 yt-1] = [xt-1 yt-1 xt-2 yt-2][ B1xx B1yx 1 0
%                                            B1xy B1yy 0 1
%                                            B2xx B2yx 0 0
%                                            B2xy B2yy 0 0 ]

CON = ones(size(FAC,1),1);
TREND = 1:size(FAC,1);
TREND = TREND';
TREND2 = TREND.^2;
TREND3 = TREND.^3;
CONS = [CON];
[CT CN] = size(CONS);


% Restricted Least Squares from the following formula: 
% B_r = B + (X'*X)\R'*(R*(X'*X)\R')\(q - R*B);

R = zeros(NF_CAN, NF*lags);
q = zeros(NF_CAN*lags,1);
for ii = 1:NF_CAN;
    for ll = 1:lags;
        R((ll-1)*NF_CAN+ii,CN+(ll-1)*(NF) + NQE+ii) = 1;
    end;
end;

for ii = 1:NQE;
    X_temp = [CONS LFAC];
    Y_temp = FAC(:,ii);
    BB_temp = (X_temp'*X_temp)\(X_temp'*Y_temp);
    BB_res(:,ii) = BB_temp + (X_temp'*X_temp)\R'*inv(R*inv(X_temp'*X_temp)*R')*(q - R*BB_temp);
end;

for ii = NQE+1:NF;
    X_temp = [CONS LFAC];
    Y_temp = FAC(:,ii);
    BB_res(:,ii) = (X_temp'*X_temp)\(X_temp'*Y_temp); 
end;

BB0 = BB_res;
ERROR = FAC - [CONS LFAC]*BB0;
[NK NN] = size(BB0);
% ---------------------------------------------


%% Benchmark Forecast: 
% *************************************************************************

% Forecast the FACTORS: 
% Set up initial data matrix: 

FAC_FF =[];
FAC_FF = [CONS(end-horz,:) FAC(end - horz,:)];

for ll = 1:lags-1;
    FAC_FF = [FAC_FF FAC(end - horz -ll,:)];
end;

% Generate large BB matrix, as in example above
BB = [BB0 zeros(NK,NN*(lags-1))];
BB(1+CN:NN*(lags-1)+CN, NN+1:end) = eye(NN*(lags-1));

ERROR_F = [ERROR zeros(size(ERROR,1),NN*(lags-1))];


% Generate Forecast:
for tt = 2:horz+1;
    FAC_F(tt,:) = [FAC_FF(tt-1,:)]*BB ;
    FAC_FF(tt,:) = [CONS(end-horz-1+tt,:) FAC_F(tt,:)];
    for ll = 1:lags;
         FAC_FF(tt,CN+(ll-1)*(NF-NQE+1) + ll*(NQE-1)) = FAC(end-horz+tt-ll,NQE-1);
         FAC_FF(tt,CN+(ll-1)*(NF-NQE) + ll*NQE) = FAC(end-horz+tt-ll,NQE);
    end;
end;

FAC_FR = FAC_F(:,1:NN);
BENCH_FAC = FAC_FR;
% ------------------------------------------------------------------------

% Transform back to variable of interest
XUS_FR = FAC_FR(:,1:NF_US+NY)*BRUS;
XCAN_FR = FAC_FR(:,:)*BRCAN;

% Adjust for standardization while generating factors
XUS_FR = XUS_FR.*repmat(std(XXUS(:,XUS_index),1),size(XUS_FR,1),1) + repmat(mean(XXUS(:,XUS_index),1),size(XUS_FR,1),1);
XCAN_FR = XCAN_FR.*repmat(std(XXCAN(:,XCAN_index),1),size(XCAN_FR,1),1) + repmat(mean(XXCAN(:,XCAN_index),1),size(XCAN_FR,1),1);

XUS_QE = FAC_FR(:,NQE).*repmat(std(XXUS(:,QE_index),1),size(XUS_FR,1),1) + repmat(mean(XXUS(:,QE_index),1),size(XUS_FR,1),1);


% Transform back to levels: US

XRUS_FR = zeros(horz+1,size(XUS_index,2));
XRUS_FR(1,:) = DUS(end-horz,XUS_index);

for j = 1:size(XUS_index,2);
for tt = 2:horz+1;
    if US_trans(XUS_index(j)) == 1;     
        XRUS_FR(tt,j) = XUS_FR(tt,j);
    elseif US_trans(XUS_index(j)) == 2; 
        XRUS_FR(tt,j) = XUS_FR(tt,j) + XRUS_FR(tt-1,j);
    elseif US_trans(XUS_index(j)) == 4; 
        XRUS_FR(tt,j) = exp(XUS_FR(tt,j));
    elseif US_trans(XUS_index(j)) == 5; 
        XRUS_FR(tt,j) = exp(XUS_FR(tt,j) + log(XRUS_FR(tt-1,j)));
    end; 
end;
end;


% Transform back to levels: CAN
XRCAN_FR = zeros(horz+1,size(XCAN_index,2));
XRCAN_FR(1,:) = DCAN(end-horz,XCAN_index);

for j = 1:size(XCAN_index,2);
for tt = 2:horz+1;
    if CAN_trans(XCAN_index(j)) == 1; 
        XRCAN_FR(tt,j) = XCAN_FR(tt,j);
    elseif CAN_trans(XCAN_index(j)) == 2; 
        XRCAN_FR(tt,j) = XCAN_FR(tt,j) + XRCAN_FR(tt-1,j);
    elseif CAN_trans(XCAN_index(j)) == 4; 
        XRCAN_FR(tt,j) = exp(XCAN_FR(tt,j));
    elseif CAN_trans(XCAN_index(j)) == 5; 
        XRCAN_FR(tt,j) = exp(XCAN_FR(tt,j) + log(XRCAN_FR(tt-1,j)));
    end; 
end;
end;


%% Counterfactual Forecast: 
% *************************************************************************

% First: Forecast the QE variable with data till 2008Q4:

for ii = 1:NQE;
    X_temp = [CONS(1:end-cdate,:) LFAC(1:end-cdate,:)];
    Y_temp = FAC(1:end-cdate,ii);
    BB_temp = (X_temp'*X_temp)\(X_temp'*Y_temp);
    BB_res(:,ii) = BB_temp + (X_temp'*X_temp)\R'*inv(R*inv(X_temp'*X_temp)*R')*(q - R*BB_temp);
end;

for ii = NQE+1:NF;
    X_temp = [CONS(1:end-cdate,:) LFAC(1:end-cdate,:)];
    Y_temp = FAC(1:end-cdate,ii);
    BB_res(:,ii) = (X_temp'*X_temp)\(X_temp'*Y_temp); 
end;

BB_C0 = BB_res;
[NK NN] = size(BB_C0);

ERROR_C0 = FAC(1:end-cdate,:) - [CONS(1:end-cdate,:) LFAC(1:end-cdate,:)]*BB_C0;
BRUS_C0 = olssvd(XRUS(1:end-cdate,:), FAC(1:end-cdate,1:NQE));
BRCAN_C0 = olssvd(XRCAN(1:end-cdate,:), [FAC(1:end-cdate,:)]);
UUS_C0 = XRUS(1:end-cdate,:) - FAC(1:end-cdate,1:NQE)*BRUS_C0;
UCAN_C0 = XRCAN(1:end-cdate,:) - FAC(1:end-cdate,:)*BRCAN_C0;
% Forecast the FACTORS: 
% Set up initial data matrix: 

FAC_C =[];
FAC_CF = [CONS(end-cdate,:) FAC(end - cdate,:)];

for ll = 1:lags-1;
    FAC_CF = [FAC_CF FAC(end - cdate -ll,:)];
end;

% Generate large BB matrix, as in example above
BB_C = [BB_C0 zeros(NK,NN*(lags-1))];
BB_C(1+CN:NN*(lags-1)+CN, NN+1:end) = eye(NN*(lags-1));


% Generate Forecast:
for tt = 2:cdate+1;
    FAC_C(tt,:) = [FAC_CF(tt-1,:)]*BB_C;
    FAC_CF(tt,:) = [CONS(end-cdate-1+tt,:) FAC_C(tt,:)];
end;

FAC_C = FAC_C(:,1:NN);
CFAC = FAC;
CFAC(end-cdate+1:end, NF_US+NY) = FAC_C(2:end,NF_US+NY);

% Second: Now use the above series as counter-factual:
% ----------------------------------------------------

CFAC_FF =[];
CFAC_FF = [CONS(end-horz,:) CFAC(end - horz,:)];

for ll = 1:lags-1;
    CFAC_FF = [CFAC_FF CFAC(end - horz -ll,:)];
end;

% Generate large BB matrix, as in example above
% BB = [BB zeros(NK,NN*(lags-1))];
% BB(1:NN*(lags-1), NN+1:end) = eye(NN*(lags-1));

% Generate Forecast:
for tt = 2:horz+1;
     CFAC_F(tt,:) = [CFAC_FF(tt-1,:)]*BB;
    CFAC_FF(tt,:) = [CONS(end-horz-1+tt,:) CFAC_F(tt,:)];
    for ll = 1:lags;
         CFAC_FF(tt,CN+(ll-1)*(NF-NQE+1) + ll*(NQE-1)) = FAC(end-horz+tt-ll,NQE-1);
         CFAC_FF(tt,CN+(ll-1)*(NF-NQE) + ll*NQE) = CFAC(end-horz+tt-ll,NQE);
    end;
end;

% Add in the forecast errors: 
CFAC_FR = CFAC_FF(:,2:NN+1);

% ------------------------------------------------------------------------

% Transform back to variable of interest

CUS_FR = CFAC_FR(:,1:NF_US+NY)*BRUS;
CCAN_FR = CFAC_FR(:,:)*BRCAN;

% Adjust for standardization while generating factors
CUS_FR = CUS_FR.*repmat(std(XXUS(:,XUS_index),1),size(XUS_FR,1),1) + repmat(mean(XXUS(:,XUS_index),1),size(XUS_FR,1),1);
CCAN_FR = CCAN_FR.*repmat(std(XXCAN(:,XCAN_index),1),size(XCAN_FR,1),1) + repmat(mean(XXCAN(:,XCAN_index),1),size(XCAN_FR,1),1);

CFAC_FR0 = CFAC_FR;
FAC_FR0 = FAC_FR;

% Transform back to levels: US

CRUS_FR = zeros(horz+1,size(XUS_index,2));
CRUS_FR(1,:) = DUS(end-horz,XUS_index);

for j = 1:size(XUS_index,2);
for tt = 2:horz+1;
    if US_trans(XUS_index(j)) == 1; 
        CRUS_FR(tt,j) = CUS_FR(tt,j);
    elseif US_trans(XUS_index(j)) == 2; 
        CRUS_FR(tt,j) = CUS_FR(tt,j) + CRUS_FR(tt-1,j);
    elseif US_trans(XUS_index(j)) == 4; 
        CRUS_FR(tt,j) = exp(CUS_FR(tt,j));
    elseif US_trans(XUS_index(j)) == 5; 
        CRUS_FR(tt,j) = exp(CUS_FR(tt,j) + log(CRUS_FR(tt-1,j)));
    end; 
end;
end;

% Transform back to levels: CAN
CRCAN_FR = zeros(horz+1,size(XCAN_index,2));
CRCAN_FR(1,:) = DCAN(end-horz,XCAN_index);

for j = 1:size(XCAN_index,2);
for tt = 2:horz+1;
    if CAN_trans(XCAN_index(j)) == 1; 
        CRCAN_FR(tt,j) = CCAN_FR(tt,j);
    elseif CAN_trans(XCAN_index(j)) == 2; 
        CRCAN_FR(tt,j) = CCAN_FR(tt,j) + CRCAN_FR(tt-1,j);
    elseif CAN_trans(XCAN_index(j)) == 4; 
        CRCAN_FR(tt,j) = exp(CCAN_FR(tt,j));
    elseif CAN_trans(XCAN_index(j)) == 5; 
        CRCAN_FR(tt,j) = exp(CCAN_FR(tt,j) + log(CRCAN_FR(tt-1,j)));
    end; 
end;
end;

% -------------------------------------------------------------------------

% Change to NET EXPORTS: 
XRUS_FR(:,10) = XRUS_FR(:,16) - XRUS_FR(:,17);
CRUS_FR(:,10) = CRUS_FR(:,16) - CRUS_FR(:,17);

XRCAN_FR(:,10) = XRCAN_FR(:,16) - XRCAN_FR(:,17);
XRCAN_FR(:,13) = XRCAN_FR(:,18) - XRCAN_FR(:,19);
CRCAN_FR(:,10) = CRCAN_FR(:,16) - CRCAN_FR(:,17);
CRCAN_FR(:,13) = CRCAN_FR(:,18) - CRCAN_FR(:,19);
 
% Transformation for figures: 
% -------------------------------------------------
PDIFCAN_R0 = 100*(XRCAN_FR - CRCAN_FR)./CRCAN_FR;
PDIFUS_R0 = 100*(XRUS_FR - CRUS_FR)./CRUS_FR;
PDIFUS_R0(:,10) = 100*(XRUS_FR(:,10) - CRUS_FR(:,10))./CRUS_FR(:,1);
PDIFCAN_R0(:,10) = 100*(XRCAN_FR(:,10) - CRCAN_FR(:,10))./CRCAN_FR(:,1);
PDIFCAN_R0(:,13) = 100*(XRCAN_FR(:,13) - CRCAN_FR(:,13))./CRCAN_FR(:,1);

DIFCAN_R0 = (XRCAN_FR - CRCAN_FR);
DIFUS_R0 = (XRUS_FR - CRUS_FR);


for jj = 1:size(XCAN_index2,2);
    NETCAN_C0(:,jj) = CRCAN_FR(:,N2+jj) - CRCAN_FR(:,N3+jj);
    NETCAN_X0(:,jj) = XRCAN_FR(:,N2+jj) - XRCAN_FR(:,N3+jj);
    DIFNET0(:,jj) = 100*(NETCAN_X0(:,jj) - NETCAN_C0(:,jj))./CRCAN_FR(:,1);
end;


%% Bootstrap:
% --------------------------------------------

[T, K] = size(FAC);
shock = zeros(K,1);
shock(NF_US+NY-1) = 1;



for rep = 1: nrep;
    
 % First, generate the small sample: 
 % --------------------------------------------------------------
    btilda = BB_C0;
    bxustilda = BRUS_C0;
    bxcantilda = BRCAN_C0;
    e = ERROR_C0;
    excan = UCAN_C0;
    exus = UUS_C0;
    
    LFAC_small = LFAC(1:end-cdate,:);
    T_small = size(LFAC_small,1);
    i = ones(T_small,1) + (T_small-1).*rand(T_small,1);
    i = round(i);
    lfyi = LFAC_small(round(1+(T_small-1)*rand),:);

    for ii = 1:T_small;
        fyi = [CONS(ii,:) lfyi]*btilda + e(i(ii),:);
        fyr(ii,:) = fyi;
        lfyi = [fyi, lfyi(:,1:end-K)];
        lfyr(ii,:) = [CONS(ii,:) lfyi];
    end;
    
    lfyr = lfyr(1:T_small-1,:);
    fyr = fyr(2:T_small,:);
    
    
    xrus = fyr(:, 1:NF_US+NY)*bxustilda + exus(i(1:end-1),:);
    bxusr = olssvd(xrus, fyr(:,1:NF_US+NY));
    xrcan = fyr(:,:)*bxcantilda + excan(i(1:end-1),:);
    bxcanr = olssvd(xrcan, fyr(:,:));
    
 
% Second: Forecast the QE variable with data till 2008Q3:
% -------------------------------------------------------------------------
 
for ii = 1:NQE;
    X_temp = lfyr(1:end-cdate,:);
    Y_temp = fyr(1:end-cdate,ii);
    BB_temp = (X_temp'*X_temp)\(X_temp'*Y_temp);
    BB_res(:,ii) = BB_temp + (X_temp'*X_temp)\R'*inv(R*inv(X_temp'*X_temp)*R')*(q - R*BB_temp);
end;

for ii = NQE+1:NF;
    X_temp = lfyr(1:end-cdate,:);
    Y_temp = fyr(1:end-cdate,ii);
    BB_res(:,ii) = (X_temp'*X_temp)\(X_temp'*Y_temp); 
end;

bb_c = BB_res;

% Forecast the FACTORS: 
% Set up initial data matrix: 

FAC_C =[];
FAC_CF = [CONS(end-cdate,:) FAC(end - cdate,:)];

for ll = 1:lags-1;
    FAC_CF = [FAC_CF FAC(end - cdate -ll,:)];
end;

% Generate large BB matrix, as in example above
BB_C = [bb_c zeros(NK,NN*(lags-1))];
BB_C(1+CN:NN*(lags-1)+CN, NN+1:end) = eye(NN*(lags-1));


% Generate Forecast:
for tt = 2:cdate+1;
    FAC_C(tt,:) = [FAC_CF(tt-1,:)]*BB_C;
    FAC_CF(tt,:) = [CONS(end-cdate-1+tt,:) FAC_C(tt,:)];
end;

FAC_C = FAC_C(:,1:NN);
CFAC = FAC;
CFAC(end-cdate+1:end, NF_US+NY) = FAC_C(2:end,NF_US+NY);


% -----------------------------------------------------------------------

% Third, re-estimate the VAR with full sample
    btilda = BB0;
    bxustilda = BRUS;
    bxcantilda = BRCAN;
    e = ERROR;
    excan = UCAN;
    exus = UUS;
   
    i = ones(T,1) + (T-1).*rand(T,1);
    i = round(i);
    lfyi = LFAC(round(1+(T-1)*rand),:);
        
    for ii = 1:T;
        fyi = [CONS(ii,:) lfyi]*btilda + e(i(ii),:);
        fyr(ii,:) = fyi;
        lfyi = [fyi, lfyi(:,1:end-K)];
        lfyr(ii,:) = [CONS(ii,:) lfyi];
    end;
    
    lfyr = lfyr(1:T-1,:);
    fyr = fyr(2:T,:);
    
    
    xrus = fyr(:, 1:NF_US+NY)*bxustilda + exus(i(1:end-1),:);
    bxusr = olssvd(xrus, fyr(:,1:NF_US+NY));
    xrcan = fyr(:,:)*bxcantilda + excan(i(1:end-1),:);
    bxcanr = olssvd(xrcan, fyr(:,:));

    
    for ii = 1:NQE;
        X_temp = lfyr;
        Y_temp = fyr(:,ii);
        BB_temp = (X_temp'*X_temp)\(X_temp'*Y_temp);
        BB_res(:,ii) = BB_temp + (X_temp'*X_temp)\R'*inv(R*inv(X_temp'*X_temp)*R')*(q - R*BB_temp);
    end;

    for ii = NQE+1:NF;
        X_temp = lfyr;
        Y_temp = fyr(:,ii);
        BB_res(:,ii) = (X_temp'*X_temp)\(X_temp'*Y_temp);
    end;
    
    br = BB_res;
    er = fyr - lfyr*br;
    Siger = er'*er/(size(er,1)-lags*K - 1);
    smatr = chol(Siger)';
    d = diag(diag(smatr));
    smatr = inv(d)*smatr;
      
    %% Benchmark Forecast: 
% *************************************************************************

% -----------------------------------------------------------------------

% Forecast the FACTORS: 
% Set up initial data matrix: 

FAC_FF =[];
FAC_FF = [CONS(end-horz,:) FAC(end - horz,:)];

for ll = 1:lags-1;
    FAC_FF = [FAC_FF FAC(end - horz -ll,:)];
end;

CFAC_FF = FAC_FF;

% Generate large BB matrix, as in example above
bbr = [br zeros(NK,NN*(lags-1))];
bbr(1+CN:NN*(lags-1)+CN, NN+1:end) = eye(NN*(lags-1));

% Generate Forecast:
for tt = 2:horz+1;
    FAC_F(tt,:) = [FAC_FF(tt-1,:)]*bbr;
    CFAC_F(tt,:) = [CFAC_FF(tt-1,:)]*bbr;
    
    FAC_FF(tt,:) = [CONS(end-horz-1+tt,:) FAC_F(tt,:)];
    CFAC_FF(tt,:) = [CONS(end-horz-1+tt,:) CFAC_F(tt,:)];
    
    for ll = 1:lags;
         FAC_FF(tt,CN+(ll-1)*(NF-NQE+1) + ll*(NQE-1)) = FAC(end-horz+tt-ll,NQE-1);
         FAC_FF(tt,CN+(ll-1)*(NF-NQE) + ll*NQE) = FAC(end-horz+tt-ll,NQE);
         CFAC_FF(tt,CN+(ll-1)*(NF-NQE+1) + ll*(NQE-1)) = FAC(end-horz+tt-ll,NQE-1);
         CFAC_FF(tt,CN+(ll-1)*(NF-NQE) + ll*NQE) = CFAC(end-horz+tt-ll,NQE);
    end;
end;


FAC_FR = FAC_F(:,1:NN);
CFAC_FR = CFAC_FF(:,2:NN+1);

% ------------------------------------------------------------------------

% Transform back to variable of interest
% XUS_FR = FAC_FR(:,1:NF_US+NY)*BRUS + UUS(end-horz:end,:);
% XCAN_FR = FAC_FR(:,NF_US+NY+1:end)*BRCAN + UCAN(end-horz:end,:);

XUS_FR = FAC_FR(:,1:NF_US+NY)*bxusr;
XCAN_FR = FAC_FR(:,:)*bxcanr;
CXUS_FR = CFAC_FR(:,1:NF_US+NY)*bxusr;
CXCAN_FR = CFAC_FR(:,:)*bxcanr;


% Adjust for standardization while generating factors
XUS_FR = XUS_FR.*repmat(std(XXUS(:,XUS_index),1),size(XUS_FR,1),1) + repmat(mean(XXUS(:,XUS_index),1),size(XUS_FR,1),1);
XCAN_FR = XCAN_FR.*repmat(std(XXCAN(:,XCAN_index),1),size(XCAN_FR,1),1) + repmat(mean(XXCAN(:,XCAN_index),1),size(XCAN_FR,1),1);

CXUS_FR = CXUS_FR.*repmat(std(XXUS(:,XUS_index),1),size(XUS_FR,1),1) + repmat(mean(XXUS(:,XUS_index),1),size(XUS_FR,1),1);
CXCAN_FR = CXCAN_FR.*repmat(std(XXCAN(:,XCAN_index),1),size(XCAN_FR,1),1) + repmat(mean(XXCAN(:,XCAN_index),1),size(XCAN_FR,1),1);
    
% Transform back to levels: US

XRUS_FR = zeros(horz+1,size(XUS_index,2));
XRUS_FR(1,:) = DUS(end-horz,XUS_index);

CRUS_FR = zeros(horz+1,size(XUS_index,2));
CRUS_FR(1,:) = DUS(end-horz,XUS_index);

for j = 1:size(XUS_index,2);
for tt = 2:horz+1;
    if US_trans(XUS_index(j)) == 1; 
        XRUS_FR(tt,j) = XUS_FR(tt,j);
        CRUS_FR(tt,j) = CXUS_FR(tt,j);
    elseif US_trans(XUS_index(j)) == 2; 
        XRUS_FR(tt,j) = XUS_FR(tt,j) + XRUS_FR(tt-1,j);
        CRUS_FR(tt,j) = CXUS_FR(tt,j) + CRUS_FR(tt-1,j);
    elseif US_trans(XUS_index(j)) == 4; 
        XRUS_FR(tt,j) = exp(XUS_FR(tt,j));
        CRUS_FR(tt,j) = exp(CXUS_FR(tt,j));
    elseif US_trans(XUS_index(j)) == 5; 
        XRUS_FR(tt,j) = exp(XUS_FR(tt,j) + log(XRUS_FR(tt-1,j)));
        CRUS_FR(tt,j) = exp(CXUS_FR(tt,j) + log(CRUS_FR(tt-1,j)));
    end; 
end;
end;

% Transform back to levels: CAN
XRCAN_FR = zeros(horz+1,size(XCAN_index,2));
XRCAN_FR(1,:) = DCAN(end-horz,XCAN_index);

CRCAN_FR = zeros(horz+1,size(XCAN_index,2));
CRCAN_FR(1,:) = DCAN(end-horz,XCAN_index);

for j = 1:size(XCAN_index,2);
for tt = 2:horz+1;
    if CAN_trans(XCAN_index(j)) == 1; 
        XRCAN_FR(tt,j) = XCAN_FR(tt,j);
        CRCAN_FR(tt,j) = CXCAN_FR(tt,j);
    elseif CAN_trans(XCAN_index(j)) == 2; 
        XRCAN_FR(tt,j) = XCAN_FR(tt,j) + XRCAN_FR(tt-1,j);
        CRCAN_FR(tt,j) = CXCAN_FR(tt,j) + CRCAN_FR(tt-1,j);
    elseif CAN_trans(XCAN_index(j)) == 4; 
        XRCAN_FR(tt,j) = exp(XCAN_FR(tt,j));
        CRCAN_FR(tt,j) = exp(CXCAN_FR(tt,j));
    elseif CAN_trans(XCAN_index(j)) == 5; 
        XRCAN_FR(tt,j) = exp(XCAN_FR(tt,j) + log(XRCAN_FR(tt-1,j)));
        CRCAN_FR(tt,j) = exp(CXCAN_FR(tt,j) + log(CRCAN_FR(tt-1,j)));
    end; 
end;
end;

for tt = 2:horz+1;
    PXRUS_FR(tt,1) = 4*(XRUS_FR(tt,1)./XRUS_FR(tt-1,1)-1);
    PCRUS_FR(tt,1) = 4*(CRUS_FR(tt,1)./CRUS_FR(tt-1,1)-1);
    PXRCAN_FR(tt,1) = 4*(XRCAN_FR(tt,1)./XRCAN_FR(tt-1,1)-1);
    PCRCAN_FR(tt,1) = 4*(CRCAN_FR(tt,1)./CRCAN_FR(tt-1,1)-1);
end;
    
% --------------------------------------
% Change to NET EXPORTS: 
XRUS_FR(:,10) = XRUS_FR(:,16) - XRUS_FR(:,17);
CRUS_FR(:,10) = CRUS_FR(:,16) - CRUS_FR(:,17);

XRCAN_FR(:,10) = XRCAN_FR(:,16) - XRCAN_FR(:,17);
XRCAN_FR(:,13) = XRCAN_FR(:,18) - XRCAN_FR(:,19);
CRCAN_FR(:,10) = CRCAN_FR(:,16) - CRCAN_FR(:,17);
CRCAN_FR(:,13) = CRCAN_FR(:,18) - CRCAN_FR(:,19);
% --------------------------------------


PDIFUS_R(:,:,rep) = 100*(XRUS_FR - CRUS_FR)./CRUS_FR;
PDIFCAN_R(:,:,rep) = 100*(XRCAN_FR - CRCAN_FR)./CRCAN_FR;
DIFUS_R(:,:,rep) = (XRUS_FR - CRUS_FR);
DIFCAN_R(:,:,rep) = (XRCAN_FR - CRCAN_FR);

PDIFUS_R(:,10,rep) = 100*(XRUS_FR(:,10) - CRUS_FR(:,10))./CRUS_FR(:,1);
PDIFCAN_R(:,10,rep) = 100*(XRCAN_FR(:,10) - CRCAN_FR(:,10))./CRCAN_FR(:,1);
PDIFCAN_R(:,13,rep) = 100*(XRCAN_FR(:,13) - CRCAN_FR(:,13))./CRCAN_FR(:,1);

for jj = 1:size(XCAN_index2,2);
    NETCAN_C(:,jj) = CRCAN_FR(:,N2+jj) - CRCAN_FR(:,N3+jj);
    NETCAN_X(:,jj) = XRCAN_FR(:,N2+jj) - XRCAN_FR(:,N3+jj);
    DIFNET(:,jj,rep) = 100*(NETCAN_X(:,jj) - NETCAN_C(:,jj))./CRCAN_FR(:,1);
end;
   
end;  

%% Generate Figures; 
% *****************************************

DIFCAN_R = sort(DIFCAN_R,3);
DIFUS_R = sort(DIFUS_R,3);
PDIFCAN_R = sort(PDIFCAN_R,3);
PDIFUS_R = sort(PDIFUS_R,3);

DIFCAN_R = DIFCAN_R(:,:,[0.16*nrep 0.5*nrep 0.84*nrep]);
DIFUS_R = DIFUS_R(:,:,[0.16*nrep 0.5*nrep 0.84*nrep]);
PDIFCAN_R = PDIFCAN_R(:,:,[0.16*nrep 0.5*nrep 0.84*nrep]);
PDIFUS_R = PDIFUS_R(:,:,[0.16*nrep 0.5*nrep 0.84*nrep]);


DIFNET = sort(DIFNET,3);
DIFNET = DIFNET(:,:,[0.16*nrep 0.5*nrep 0.84*nrep]);

%set up dates for figures
horiDate = datenum('09-01-2008'); 
endDate = datenum('09-01-2013');
tData = linspace(horiDate,endDate,21);
    a = (endDate-horiDate)/2;
    middate1 = horiDate+a;

XRED = zeros(size(tData));

% Generate Counterfactual DIFFERENCE figure: US
% ---------------------------------------------
figure('Position', [100 100 800 900]);
for j = 1: 5*3;
    subplot(5,3,j);
    plot(tData,PDIFUS_R(:,j,1),'--b',tData,PDIFUS_R0(:,j),'-k',tData,PDIFUS_R(:,j,3),'--b',tData, XRED,'-r', 'LineWidth',2);
        set(gca,'XTick',[horiDate middate1 endDate])
        datetick('x','yyqq','keepticks')
    axis tight;
    title(USVARNAM(j));
end;

for j = [3,8,14,15];
    subplot(5,3,j);
    plot(tData,DIFUS_R(:,j,1),'--b',tData,DIFUS_R0(:,j),'-k',tData,DIFUS_R(:,j,3),'--b',tData, XRED,'-r', 'LineWidth',2);
        set(gca,'XTick',[horiDate middate1 endDate])
        datetick('x','yyqq','keepticks')
    axis tight;
    title(USVARNAM(j));
end;

set(figure(1),'PaperPositionMode', 'auto') 
saveas(gcf,'MS14-430Fig2','eps');
%save2pdf('MS14-430Fig2'); 


% Generate Counterfactual DIFFERENCE figure: CAN
% ---------------------------------------------

figure('Position', [1000 100 800 900]);
for j = 1: 5*3;
    subplot(5,3,j);
    plot(tData,PDIFCAN_R(:,j,1),'--b',tData,PDIFCAN_R0(:,j),'-k',tData,PDIFCAN_R(:,j,3),'--b', tData, XRED,'-r','LineWidth',2);
        set(gca,'XTick',[horiDate middate1 endDate])
        datetick('x','yyqq','keepticks')
    axis tight;
    title(CANVARNAM(j));
end;

for j = [3,8,11,15];
    subplot(5,3,j);
    plot(tData,DIFCAN_R(:,j,1),'--b',tData,DIFCAN_R0(:,j),'-k',tData,DIFCAN_R(:,j,3),'--b', tData, XRED,'-r','LineWidth',2);
        set(gca,'XTick',[horiDate middate1 endDate])
        datetick('x','yyqq','keepticks')
    axis tight;
    title(CANVARNAM(j));
end;

set(figure(2),'PaperPositionMode', 'auto') 
saveas(gcf,'MS14-430Fig3','eps');
%save2pdf('MS14-430Fig3'); 


